'use strict';

var DefaultOptions = require('./default-options');

var _require = require('./../helpers'),
    isObject = _require.isObject,
    isStringOrArray = _require.isStringOrArray;

var userOptionHandlers = require('./user-option-handlers');

var pluginOptions = new DefaultOptions();

exports.run = function (compiler) {
    var userOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    verify(userOptions);
    compiler.plugin('environment', function () {
        update(this.options, userOptions);
    });
};

exports.get = function (name, defaultValue) {
    return pluginOptions.has(name) ? pluginOptions.get(name) : defaultValue;
};

function verify(options) {
    if (!isObject(options)) {
        throw new Error('Options should be an object.');
    }
    if (options.entries && !isStringOrArray(options.entries)) {
        throw new Error('Entries option should be a String or Array');
    }
    if (options.whitelist && !isStringOrArray(options.whitelist)) {
        throw new Error('Whitelist option should be a String or Array');
    }
    if (options.aliases && !Array.isArray(options.aliases)) {
        throw new Error('Aliases option should be an Array');
    }
}

function update(configuration, userOptions) {
    override(processOptions(configuration, userOptions, {
        entries: userOptionHandlers.alignEntries,
        whitelist: userOptionHandlers.parseWhitelist,
        strict: Boolean,
        aliases: function aliases(_aliases) {
            return _aliases.filter(Boolean);
        }
    }));
}

function processOptions(configuration, options) {
    var handlers = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    var result = Object.create(null);
    Object.keys(options).forEach(function (optionName) {
        var optionValue = options[optionName];
        if (optionName in handlers) {
            var handler = handlers[optionName];
            if (typeof handler === 'function') {
                optionValue = handler(optionValue, configuration);
            } else {
                optionValue = handler;
            }
        }
        result[optionName] = optionValue;
    });

    return result;
}

function override(options) {
    Object.keys(options).forEach(function (name) {
        if (pluginOptions.has(name)) {
            pluginOptions.set(name, options[name]);
        }
    });
}