'use strict';

var defaultOptions = require('./defaults');

var _require = require('./../helpers'),
    isObject = _require.isObject,
    isStringOrArray = _require.isStringOrArray,
    ensureArray = _require.ensureArray;

var options = Object.create(null);

init({});

function init(compiler) {
    var rawOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    verify(rawOptions);
    update(compiler.options, rawOptions);
}

exports.init = init;

exports.get = function (name, defaultValue) {
    if (name in options) {
        return options[name];
    }

    return defaultValue;
};

function verify(options) {
    if (!isObject(options)) {
        throw new Error('Options should be an object.');
    }
    if (options.entries && !isStringOrArray(options.entries)) {
        throw new Error('Entries option should be a String or Array');
    }
    if (options.whitelist && !isStringOrArray(options.whitelist)) {
        throw new Error('Whitelist option should be a String or Array');
    }
    if (options.aliases && !Array.isArray(options.aliases)) {
        throw new Error('Aliases option should be an Array');
    }
}

function update(configuration, rawOptions) {
    var overrideDefaults = processOptions(configuration, rawOptions, {
        entries: alignEntries,
        whitelist: parseWhitelist,
        strict: Boolean,
        aliases: function aliases(_aliases) {
            return _aliases.filter(Boolean);
        }
    });
    options = defaultOptions.get(overrideDefaults);
}

function processOptions(configuration, rawOptions) {
    var handlers = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    var result = Object.create(null);
    Object.keys(rawOptions).forEach(function (optionName) {
        var optionValue = rawOptions[optionName];
        if (optionName in handlers) {
            var handler = handlers[optionName];
            if (typeof handler === 'function') {
                optionValue = handler(optionValue, configuration);
            } else {
                optionValue = handler;
            }
        }
        result[optionName] = optionValue;
    });

    return result;
}

function alignEntries(optionsEntries, configuration) {
    var resultEntries = [];
    var configurationEntry = configuration.entry;
    if (isObject(configurationEntry)) {
        resultEntries = optionsEntries;
    }

    return ensureArray(resultEntries);
}

function parseWhitelist(whitelist) {
    if (typeof whitelist === 'string') {
        if (whitelist.indexOf(',') > -1) {
            whitelist = whitelist.replace(/\s/g, '').split(',');
        } else {
            whitelist = whitelist.split(/\s+/);
        }
    }

    return whitelist.filter(Boolean);
}