'use strict';

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var babelCore = require('babel-core');
var RawModule = require('webpack/lib/RawModule');
var SingleEntryDependencyWrapper = require('../SingleEntryDependencyWrapper');
var _Symbol = require('symbol');
var NullFactory = require('webpack/lib/NullFactory');

var moduleIdentifierSymbol = _Symbol('topLevelModuleIdentifier');

module.exports = function (compiler, pluginOptions) {
    compiler.plugin('compilation', function (compilation) {
        inject(compilation, pluginOptions);
    });
};

function inject(compilation, pluginOptions) {
    compilation.dependencyFactories.set(SingleEntryDependencyWrapper, new NullFactory());
    compilation.plugin('seal', function () {
        var _this = this;

        getChunks(this, pluginOptions.entries).forEach(function (chunk) {
            return injectHelpers(_this, chunk.module, pluginOptions.whitelist);
        });
    });
}

function getChunks(compilation, onlyEntries) {
    var chunks = compilation.preparedChunks;
    if (onlyEntries.length < 1) {
        return chunks;
    }

    return chunks.filter(function (chunk) {
        return onlyEntries.indexOf(chunk.name) > -1;
    });
}

function injectHelpers(compilation, topLevelModule, babelHelpersWhitelist) {
    var dependency = getDependency(topLevelModule);

    var _addModule = addModule(compilation, createHelpersModule(dependency, babelHelpersWhitelist)),
        _addModule2 = _slicedToArray(_addModule, 2),
        module = _addModule2[0],
        fromCache = _addModule2[1];

    link(topLevelModule, module, dependency);
    if (!fromCache) {
        topLevelModule.dependencies.unshift(dependency);
        compilation.buildModule(module, false, topLevelModule, [dependency], function (error) {
            if (error) {
                throw new Error('Helpers module is not build.');
            }
        });
    }
}

function addModule(compilation, module) {
    var added = compilation.addModule(module);
    var cached = added !== true;
    if (added === false) {
        module = compilation.getModule(module);
    } else if (added instanceof RawModule) {
        module = added;
    }

    return [module, cached];
}

function getDependency(module) {
    return findInjectedDependency(module) || createDependency(module);
}

function findInjectedDependency(module) {
    return module.dependencies.filter(function (dependency) {
        return dependency instanceof SingleEntryDependencyWrapper;
    })[0];
}

function createDependency(module) {
    var dependency = new SingleEntryDependencyWrapper('babel-loader-external-helpers');
    dependency.loc = module.name + ":" + (100000 - 1);
    dependency[moduleIdentifierSymbol] = module.identifier();

    return dependency;
}

function createHelpersModule(dependency, whitelist) {
    return new RawModule(buildBabelHelpers(whitelist), getModuleIdentifier(dependency));
}

function buildBabelHelpers(whitelist) {
    if (whitelist.length) {
        return babelCore.buildExternalHelpers(whitelist);
    }

    return babelCore.buildExternalHelpers();
}

function getModuleIdentifier(dependency) {
    return 'generated babel-helpers ' + dependency[moduleIdentifierSymbol];
}

function link(topLevelModule, module, dependecy) {
    module.issuer = topLevelModule.identifier();
    dependecy.module = module;
    module.addReason(topLevelModule, dependecy);
}